%% batch example of the "hybrid "connICA (Amico & Goñi, Network Neuroscience 2018)
%% Enrico Amico & Joaquín Goñi, Purdue University
%% Thanks to Jonathan Wirsich (Beckman Institute, UIUC) for insightful comments and discussions
%% version 1.0. April 10, 2018
%
%% PLEASE CITE US!
% If you are using this code for your research, please kindly cite us:
%% 1) Enrico Amico and Joaquín Goñi. Mapping hybrid Functional-structural connectivity traits in the human connectome. Network
%% Neuroscience, (in press), 2018. https://doi.org/10.1162/netn_a_00049
%
%% 2) Mapping the functional connectome traits of levels of consciousness. Amico, E. ... & Goñi, J. (2017). NeuroImage, 148, 201-211.
%% http://www.sciencedirect.com/science/article/pii/S1053811917300204
%
%% IMPORTANT: FastICA Matlab package IS NEEDED!
% Please download FastICA package
% https://research.ics.aalto.fi/ica/fastica/code/dlcode.shtml

%% initialize environment
close all;
clearvars
clc

%% changing default fontsize
fontsize = 18;
set(0,'DefaultAxesFontName','Times New Roman');
set(0,'DefaultAxesFontSize',fontsize-2);

set(0,'DefaultTextFontname','Times New Roman');
set(0,'DefaultTextFontSize',fontsize);

%% Load FC data (functional connectivity) and SC data (structural connectivity). This is a sample data.
%  dimensions are brain regions x brain regions x subjects
load FC.mat;
load SC.mat;

%% Configuration
addpath(genpath('FastICA_25')); %add FastICA folder to path. Please check folder_name and update if needed.
% ConnICA params
configs.numRegions = 164; % aparc2009 parcellation
configs.mask_ut = triu(true(configs.numRegions,configs.numRegions),1); % upper triangular mask, for FC data
configs.mask_lt = tril(true(configs.numRegions,configs.numRegions),-1); % lower triangular mask, for SC data
% Note: choice of the mask is purely conventional. However, the reshaping back to matrix form needs to be performed accordingly

configs.numConn = size(FC,3);%number of connectomes (subjects or sessions)

%% Compute group average SC mask of connected edges (see for details Amico & Goni, Network Neuroscience 2018)
mask_nonzeroSC = true(configs.numRegions,configs.numRegions);
for i=1:configs.numConn
     SC_i = SC(:,:,i);
     mask_nonzeroSC = mask_nonzeroSC & (SC_i>0);
end


%% create connICA hybrid input matrix (nSubj x (2 x n FC-SC edges))
connICA_hybrid = zeros(configs.numConn,2*nnz(configs.mask_ut));
disp('Creating hybrid connICA matrix...')
for i=1:configs.numConn
    aux = FC(:,:,i);
    FC_part = aux(configs.mask_ut);
    aux = SC(:,:,i);
    %% Compute SC "correlation node to node" (see Amico & Goñi, Network Neuroscience 2018)
    % Note: other data normalization can be used here
    SC_corr = f_compute_SC_corr(aux);
    SC_part = SC_corr(configs.mask_lt);
    % concatenate FC and SC                
    connICA_hybrid(i,:) = [FC_part' SC_part'];
end
disp('Done.');
%% For the SC part, consider only structurally connected edges when extracting hybrid traits
%% (see for details Amico & Goñi, Network Neuroscience 2018)
connICA_hybrid(:,[false(1,nnz(configs.mask_ut)) ~mask_nonzeroSC(configs.mask_lt)'])= []; 

%% FastICA params
configs.epsilon = 0.0001; % epsilon default is 0.0001
configs.maxNumIterations = 1000; % maxNumIterations default is 1000
configs.maxFinetune = 100; % maxFinetune default is 100
configs.numRuns = 100; %100; %run FastICA several time for robust comps extraction  
configs.numOfIC = 5; % number independent components (explore different range)    

flags.PCA = 1; % if 1, perform PCA before ICA
configs.PCAthr = 0.90; % Explained variance threshold for PCA. Feel free to explore 0.75 or .80
% perform PCA before ICA
if flags.PCA==1
    disp('running PCA compression before ICA...');
    numFCs = size(connICA_hybrid,1);
    numPCAComps = size(connICA_hybrid,1);
    [~, ~, latent] = pca(connICA_hybrid','NumComponents',numPCAComps); 
    variance = cumsum(latent)./sum(latent); 
    variance = variance(1:numPCAComps); %explained variance with the selected num of PCA comps
    numPCAComps = find(variance>=configs.PCAthr,1);
    disp('# PCA comps retained:');
    disp(numPCAComps);
    if numPCAComps<configs.numOfIC
        error('Error: PCA comps retained lower than number of requested ICA comps! Check the configs.numOfIC and try again');
    end
    if numPCAComps==configs.numOfIC
        warning(sprintf('Warning: number of PCA comps retained equal to the number of requested ICA comps.\n Please note that, in our experience, sometimes FastICA does not reach convergence under this configuration')); %#ok<SPWRN>
    end
    
    %figure, plot(1:numFCs,variance,'ok'); % uncomment this line if you want to see the variance explained plot
    [COEFFS, SCORE, latent] = pca(connICA_hybrid','NumComponents',numPCAComps);     
    PCA_clean_matrix = SCORE * COEFFS'; % PCA reconstructed demeaned data
    PCA_clean_matrix = bsxfun(@plus, PCA_clean_matrix,mean(connICA_hybrid,2)'); % plug the mean back
    PCA_clean_matrix = PCA_clean_matrix'; %back to subjects x edges form
    connICA_hybrid = PCA_clean_matrix;
    disp('Done.');
end


icasig = nan(size(connICA_hybrid,2),configs.numOfIC,configs.numRuns);
A = nan(size(connICA_hybrid,1),configs.numOfIC,configs.numRuns);
for i=1:configs.numRuns
    [icasig_onerun,A_onerun,~] = fastica(connICA_hybrid,'approach','symm','numOfIC',configs.numOfIC,'verbose','off',...
        'epsilon',configs.epsilon,'maxNumIterations',configs.maxNumIterations,...
        'maxFinetune',configs.maxFinetune);%running fastica
    A(:,:,i) = single(A_onerun);% connICA weights
    icasig(:,:,i) = single(icasig_onerun'); % connICA single run traits
    if mod(i,25)==0
        disp(sprintf('%d runs out of %d',i,configs.numRuns)); %#ok<DSPS>
    end
end
%% robust traits extraction criteria (see Amico et al., NeuroImage 2017 and Amico & Goñi, Network Neuroscience 2018)
configs.minFreq = 0.75; % minimum frequency required 
configs.corrMin = 0.75; % correlation between FC_traits
configs.corrMin_A = 0.75; % correlation between subject weights, set it to 0 if bootstrapping subjects across numRuns 
[comp_match_run1, freq] = run_robust_connICA_fast(A,icasig,configs); % comp_mach_run1 stores the robust component ID per connICA run
aux=(freq>configs.minFreq);
RC_Index=find(aux(:,1)); % RC_Index == Robust Components Index
disp(RC_Index)
if isempty(RC_Index)
    error('No Robust FC traits found!');
end
%% Put back the robust traits in matrix form
RC = struct;
for t=1:length(RC_Index)
    compIndex=RC_Index(t); % choose the component(order of component) we want to look at 
    figure,
    icasig_comp = nan(configs.numRuns,size(connICA_hybrid,2));
    a0 = A(:,comp_match_run1(compIndex,1),1); %this is used as reference so that the same component across runs is always positively correlated (not inverted)
    weights = nan(size(A,1),configs.numRuns);
    for i=1:configs.numRuns
        if comp_match_run1(compIndex,i)>0
            a = A(:,comp_match_run1(compIndex,i),i);
            icasig_comp_one = squeeze(icasig(:,comp_match_run1(compIndex,i),i));
            if corr(a,a0)>0
                plot(a); hold on;
                icasig_comp(i,:) = icasig_comp_one;
            else
                plot(-a); hold on;
                a= -a;
                icasig_comp(i,:) = -icasig_comp_one;
            end
            weights(:,i) = a;
        end
    end
    ylabel('weights');
    xlabel('subjects');
    title(sprintf('connICA comp %d',compIndex));
    %% Plot hybrid traits
    RC(t).hybrid_vector = nanmean(icasig_comp); %avg per column (across runs) 
    RC(t).FC_vector = [];
    RC(t).FC_vector = RC(t).hybrid_vector(1:nnz(configs.mask_ut));
    RC(t).FC_matrix = zeros(configs.numRegions,configs.numRegions);
    RC(t).FC_matrix(configs.mask_ut)= RC(t).FC_vector; %fill upper triangular (FC part)
    RC(t).FC_matrix = RC(t).FC_matrix + (RC(t).FC_matrix'); %symmetrize matrix
    RC(t).SC_vector = [];
    RC(t).SC_vector =  RC(t).hybrid_vector(nnz(configs.mask_ut)+1:end);
    RC(t).SC_matrix = zeros(configs.numRegions,configs.numRegions);
    RC(t).SC_matrix(configs.mask_lt & mask_nonzeroSC)= RC(t).SC_vector; %fill lower triangular (SC part)
    RC(t).SC_matrix = RC(t).SC_matrix + (RC(t).SC_matrix'); %symmetrize matrix    
    RC(t).weights = weights; % Hybrid trait vector of weights, per run    
    
    figure; 
    subplot(1,2,1); imagesc(RC(t).FC_matrix,[-3,3]); colormap jet; colorbar; axis square;
    set(gca,'xtick',[]); set(gca,'ytick',[]); xlabel('regions'); ylabel('regions');
    title('FC part')
    subplot(1,2,2); imagesc(RC(t).SC_matrix,[-3,3]); colormap jet; colorbar; axis square;
    set(gca,'xtick',[]); set(gca,'ytick',[]); xlabel('regions'); ylabel('regions');
    title('SC part')
    suptitle(sprintf('connICA hybrid trait %d',compIndex));
end
